<?php

/**
 * افزونه پیشرو پست - چابک
 *
 * Plugin Name: افزونه پست چابک
 * Plugin URI: #
 * Description: افزونه همگام سازی ووکامرس و چابک (پیشرو پست)
 * Version: 1.0
 * Author: Mehran Yavari
 * Text Domain: chabokwc
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly
add_action('admin_notices', function () {
    if (!is_plugin_active('pepro-ultimate-invoice/pepro-ultimate-invoice.php') and current_user_can('activate_plugins')) {
?>
        <div class="notice notice-error is-dismissible">
            <p>لطفا برای چاپ فاکتور افزونه pepro ultimate invoice را نصب و فعال کنید! </p>
        </div>
        <?php }
});

class Chabok_WC
{
    public function __construct()
    {
        ob_start();
        session_start();

        $this->ajax_get_state();

        if (get_option('wc_chabok_shipping_method_enabled') == 'yes') {
            require_once 'chabock-shipping.php';
        }
        add_filter('woocommerce_settings_tabs_array', array($this, 'add_settings_tab'), 50);
        add_action('woocommerce_settings_tabs_chabok_settings', array($this, 'settings_tab'));
        add_action('woocommerce_update_options_chabok_settings', array($this, 'update_settings'));


        add_action('woocommerce_thankyou', array($this, 'send_order_to_chabok'));
        add_action("wp_ajax_chabok_ajax", array($this, "chabok_ajax"));
        add_action("wp_ajax_nopriv_chabok_ajax", array($this, "chabok_ajax"));
        add_action('woocommerce_admin_order_data_after_billing_address', array($this, 'show_tracking_code_in_order'), 10, 1);

        add_action('woocommerce_review_order_before_submit', array($this, 'ajax_get_city'));

        add_action("wp_ajax_get_quote", array($this, "get_quote"));
        add_action("wp_ajax_nopriv_get_quote", array($this, "get_quote"));

        add_action("wp_ajax_get_cities", array($this, "get_cities"));
        add_action("wp_ajax_nopriv_get_cities", array($this, "get_cities"));

        add_action("admin_footer", array($this, "admin_scripts"));
    }


    /**
     * add settings tab in woocommerce settings
     */
    public function add_settings_tab($settings_tabs)
    {
        $settings_tabs['chabok_settings'] = 'تنظیمات چابک';
        return $settings_tabs;
    }
    /**
     * settings tab content chabock
     */
    public function settings_tab()
    {
        woocommerce_admin_fields($this->get_settings());
    }

    public function update_settings()
    {
        woocommerce_update_options($this->get_settings());
    }
    public function get_settings()
    {
        $settings = array(
            'section_title' => array(
                'name' => 'تنظیمات چابک',
                'type' => 'title',
                'desc' => '',
                'id' => 'wc_chabok_settings_section_title'
            ),
            'chabok_username' => array(
                'name' => 'نام کاربری چابک:',
                'desc' => 'نام کاربری ورود به سامانه https://ppt.chabok.app/ را وارد نمایید.',
                'type' => 'text',
                'id' => 'wc_chabok_username'
            ),
            'chabok_password' => array(
                'name' => 'رمز عبور چابک:',
                'desc' => 'رمز عبور ورود به سامانه https://ppt.chabok.app/ را وارد نمایید.',
                'type' => 'text',
                'id' => 'wc_chabok_password'
            ),
            'chabok_api_address' => array(
                'name' => 'آدرس Api:',
                'type' => 'text',
                'id' => 'wc_chabok_api',
                'placeholder' => 'https://api-XXX.chabok.app'
            ),
			'chabok_services' => array(
                'name' => 'نوع سرویس:',
                'type' => 'select',
                'id' => 'wc_chabok_services',
                'options'       => $this->get_services()
            ),
            'chabok_shipping_method_enabled' => array(
                'name'     => 'فعالسازی روش حمل و نقل چابک',
                'desc'     => 'درصورتیکه این گزینه را فعال کنید؛ روش حمل و نقل چابک به ووکامرس افزوده می شود و می توانید آن را مدیریت کنید و در غیر اینصورت در تمامی روش های حمل و نقل سفارش به چابک ارسال می شود.',
                'type' => 'checkbox',
                'default' => 'no',
                'id' => 'wc_chabok_shipping_method_enabled'
            ),
            'chabok_storename' => array(
                'name' => 'نام فروشگاه شما',
                'type' => 'text',
                'id' => 'wc_chabok_storename'
            ),
            'chabok_store_state' => array(
                'name' => 'استان محل فروشگاه',
                'type' => 'select',
                'id' => 'wc_chabok_store_state',
                'class' => 'wc-enhanced-select-nostd',
                'options'       => $this->get_states()
            ),
            'chabok_store_city' => array(
                'name' => 'شهر محل فروشگاه',
                'type' => 'select',
                'id' => 'wc_chabok_store_city',
                'class' => 'wc-enhanced-select-nostd',
                'options'       => $this->get_cities(),
            ),
            'chabok_store_city_name' => array(
                'type' => 'text',
                'class' => 'hidden',
                'id' => 'wc_chabok_store_city_name',
            ),
            'chabok_store_address' => array(
                'name' => 'آدرس فروشگاه',
                'type' => 'text',
                'id' => 'wc_chabok_store_address',
                'placeholder' => 'تهران، میدان آزادی'
            ),
            'chabok_store_postcode' => array(
                'name' => 'کد پستی فروشگاه',
                'type' => 'text',
                'id' => 'wc_chabok_store_postcode',
                'placeholder' => '1234567891'
            ),
            'chabok_storeemail' => array(
                'name' => 'ایمیل فروشگاه شما',
                'type' => 'text',
                'id' => 'wc_chabok_storeemail'
            ),
            'chabok_usertell' => array(
                'name' => 'تلفن فروشگاه شما',
                'type' => 'text',
                'id' => 'wc_chabok_usertell'
            ),
            'section_end' => array(
                'type' => 'sectionend',
                'id' => 'wc_chabok_settings_section_end'
            )
        );
        return apply_filters('wc_chabok_settings', $settings);
    }
    public function get_states()
    {
        $request = file_get_contents('https://api-ppt.chabok.app/get_state');
        $request_array = json_decode($request, true);
        $states = $request_array['objects']['state'];
        $states_name = array_column($states, 'name');
        return $states_name;
    }
	public function get_services()
    {
        $request = file_get_contents('http://api.krch.ir/v1/get_service');
        $request_array = json_decode($request, true);
        $services = $request_array['objects']['state'];
        $services_name = array_column($services, 'name');
        return $services_name;
    }
    public function ajax_get_state()
    {
        add_action('admin_footer', function () {
        ?>
            <script>
                (function($) {
                    $('#wc_chabok_store_state').change(function() {
                        var option = "";
                        $('#wc_chabok_store_city').html(option);
                        $("#wc_chabok_store_city").prop("disabled", true);
                        var state = $(this).find(':selected').val();
                        var no = parseInt(state) + 1;
                        $.ajax({
                            url: '<?php echo admin_url('admin-ajax.php'); ?>',
                            type: "post",
                            dataType: "json",
                            data: {
                                action: "get_cities",
                                no: no,
                            },
                            success: function(response) {
                                response.forEach(element => {
                                    option += "<option value='" + element['name'] + "'>" + element["name"] + "</option>";
                                });

                                $("#wc_chabok_store_city").prop("disabled", false);
                                $('#wc_chabok_store_city').append(option);
                                var city_name = $('#wc_chabok_store_city_name').val();
                                $('#wc_chabok_store_city option[value="' + city_name + '"]').prop('selected', true);
                                $('#wc_chabok_store_city').change()
                            }
                        });
                    });

                })(jQuery);
            </script>
        <?php
        });
    }
    public function get_cities()
    {
        $no = $_POST['no'] ? $_POST['no'] : '8';
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://api-ppt.chabok.app/get_city',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_POSTFIELDS => array('input' => '{"state":{"no":' . $no . '}}'),

            CURLOPT_HTTPHEADER => array(
                "app-auth: aW9zX2N1c3RvbWVyX2FwcDpUUFhAMjAxNg==",
                "content-type: multipart/form-data;charset=utf-8",
                'Cookie: PHPSESSID=0uo8kk84t6is4h3u11qbuuh8d6'
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        $f = json_decode($response, true);
        $x = $f["objects"]["city"];
        $cities = array_column($x, 'name');
        if (isset($_POST['no'])) {
            wp_die(json_encode($x));
        } else {
            return $cities;
        }
    }
    public function find_city($city_name)
    {
        $request = file_get_contents('https://api-ppt.chabok.app/get_city');
        $request_array = json_decode($request, true);
        $cites = $request_array['objects']['city'];
        $city_col = array_search($city_name, array_column($cites, 'name'));
        return $cites[$city_col]['no'];
    }

    public function ajax_get_city()
    {
        add_action('wp_footer', function () {
        ?>
            <script>
                (function($) {
                    if ($('#billing_city').is("select")) {
                        $('#billing_city').change(function() {
                            var city = $(this).find(':selected').text();
                            $.ajax({
                                url: '<?php echo admin_url('admin-ajax.php'); ?>',
                                type: "post",
                                dataType: "json",
                                data: {
                                    action: "get_quote",
                                    city: city,
                                },
                                success: function(response) {
                                    $(document.body).trigger("update_checkout");
                                }
                            });
                        });
                    }else{
                        $('#billing_city').blur(function() {
                            var city = $(this).val();
                            $.ajax({
                                url: '<?php echo admin_url('admin-ajax.php'); ?>',
                                type: "post",
                                dataType: "json",
                                data: {
                                    action: "get_quote",
                                    city: city,
                                },
                                success: function(response) {
                                    $(document.body).trigger("update_checkout");
                                }
                            });
                        });
                    }
                })(jQuery);
            </script>
        <?php
        });
    }

    /**
     * send data in order recived ajax
     */
    public function send_order_to_chabok($order_id)
    {
        $current_shipping_method = WC()->session->get('chosen_shipping_methods')[0];
        $is_chabok_shipping_method = explode(':', $current_shipping_method)[0];
        if ((get_option('wc_chabok_shipping_method_enabled') == 'yes' && $is_chabok_shipping_method == 'chabok_shipping_method') || get_option('wc_chabok_shipping_method_enabled') == 'no') {
            $order = wc_get_order($order_id);
            $order_data = $order->get_data();

            // order date
            $order_date_created = $order_data['date_created']->date('Y-m-d');
            // order total price
            $order_total = $order_data['total'];
            // order weight
            $order_items  = $order->get_items();
            $total_qty    = 0;
            $total_weight = 0;
            $categories = '';
            foreach ($order_items as $item_id => $product_item) {
                //$product         = $product_item->get_product();
                $product_id    = $product_item['product_id'];
                $product = wc_get_product($product_id);
                if (!$product) continue;
                $product_weight  = $product->get_weight() ? $product->get_weight() : 1;
                $quantity        = $product_item->get_quantity();
                $total_qty      += $quantity;
                $total_weight   += floatval($product_weight * $quantity);
                $terms = get_the_terms($product->get_id(), 'product_cat');

                $categories .= reset($terms)->name . ' / ';
            }
            // order first and last name
            $order_fullname = $order_data['billing']['first_name'] . ' ' . $order_data['billing']['last_name'];

            // order city
            $order_city = $order_data['billing']['city'];

            $city_no = $this->find_city($order_city);

            // order phone
            $order_phone = $order_data['billing']['phone'];
            //order email
            $order_email = $order_data['billing']['email'];
            // order address
            $order_address = $order_data['billing']['address_1'];
            // order postcode
            $order_postcode = $order_data['billing']['postcode'];
			if(get_option('woocommerce_weight_unit')=='kg'){
				$weight=$total_weight;
			}else{
				$weight=$total_weight/1000;
			}
        ?>
            <script>
                (function($) {
                    // Send request
                    $.ajax({
                        url: '<?php echo admin_url('admin-ajax.php'); ?>',
                        type: "post",
                        dataType: "json",
                        data: {
                            action: "chabok_ajax",
                            reference: <?= json_encode($order_id); ?>,
                            date: <?= json_encode($order_date_created); ?>,
                            inv_value: <?= json_encode($order_total); ?>,
                            weight: <?= json_encode($weight); ?>,
                            person: <?= json_encode($order_fullname); ?>,
                            city_no: <?= json_encode($city_no); ?>,
                            mobile: <?= json_encode($order_phone); ?>,
                            email: <?= json_encode($order_email); ?>,
                            address: <?= json_encode($order_address); ?>,
                            postcode: <?= json_encode($order_postcode); ?>,
                            content: <?= json_encode($categories); ?>,
                        },
                    });
                })(jQuery);
            </script>
        <?php
        }
    }

    function chabok_ajax()
    {

        ob_start();
		$api=get_option('wc_chabok_api');
        $username = get_option('wc_chabok_username');
        $password = get_option('wc_chabok_password');
        $reference = $_POST['reference'] . '111' . rand(99, 999);
        $order_id = $_POST['reference'];
        $date = $_POST['date'];
        $value = $_POST['inv_value'];
        $storename = get_option('wc_chabok_storename');
        $storeemail = get_option('wc_chabok_storeemail');
        $storecity5 = $this->find_city(get_option('wc_chabok_store_city_name'));
        $usertell = get_option('wc_chabok_usertell');
        $store_address = get_option('wc_chabok_store_address');
        $person = $_POST['person'];
        $city_no = $_POST['city_no'];
        $mobile = $_POST['mobile'];
        $email = $_POST['email'];
        $address = $_POST['address'];
        $postcode = $_POST['postcode'];
        $weight = $_POST['weight'];
        $content = $_POST['content'];
        $desc1 = " ";
        $title1 = " ";

		if(get_option('wc_chabok_storeemail')=='0'){
			$service="1";
		}elseif(get_option('wc_chabok_storeemail')=='1'){
			$service="6";
		}elseif(get_option('wc_chabok_storeemail')=='2'){
			$service="35";
		}

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $api."/bulk_import",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_POSTFIELDS => array('input' => '{"user":{"username":"' . $username . '","password":"' . $password . '"},"bulk":[{"cn":{"reference":"' . $reference . '","date":"' . $date . '","assinged_pieces":"1","service":"'.$service.'","value":"' . $value . '","inv_value":0,"payment_term":"0","weight":"' . $weight . '","content":"' . $content . '","note":""},"sender":{"person":"' . $storename . '","city_no":"' . $storecity5 . '","telephone":"' . $usertell . '","mobile":"' . $usertell . '","email":"' . $storeemail . '","address":"' . $store_address . '"},"receiver":{"person":"' . $person . '","company":"","city_no":"' . $city_no . '","telephone":"' . $mobile . '","mobile":"' . $mobile . '","email":"' . $email . '","address":"' . $address . '","postcode":"' . $postcode . '"}}]}'),

            CURLOPT_HTTPHEADER => array(
                "app-auth: aW9zX2N1c3RvbWVyX2FwcDpUUFhAMjAxNg==",
                "content-type: multipart/form-data;charset=utf-8",
                'Cookie: PHPSESSID=0uo8kk84t6is4h3u11qbuuh8d6'
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        $f = json_decode($response, true);
        $tracking = $f["objects"]["result"][0]["tracking"];
        update_post_meta($order_id, 'order_tracking_chabok', $tracking);

        return $tracking;
    }
    public function get_quote()
    {
        $origin = $this->find_city(get_option('wc_chabok_store_city_name'));
        $destination = $this->find_city($_POST['city']);
        $value = WC()->cart->subtotal;
        $weight = WC()->cart->cart_contents_weight != 0 ? WC()->cart->cart_contents_weight : '1';
        $username = get_option('wc_chabok_username');
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://api.krch.ir/v1/get_quote',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_POSTFIELDS => array('input' => '{"order":{"origin":"' . $origin . '","destination":"' . $destination . '","weight": "' . $weight . '","value":"' . $value . '","method": "1","sender_code": "' . $username . '","receiver_code":"1000","cod":"0"}}'),

            CURLOPT_HTTPHEADER => array(
                "app-auth: aW9zX2N1c3RvbWVyX2FwcDpUUFhAMjAxNg==",
                "content-type: multipart/form-data;charset=utf-8",
                'Cookie: PHPSESSID=0uo8kk84t6is4h3u11qbuuh8d6'
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        $f = json_decode($response, true);
        $price = $f["objects"]["order"]["quote"];
        if (get_woocommerce_currency() == 'IRT') {
            $price = intval($price) / 10;
        }
        $_SESSION['chabock_price'] = $price;
        setcookie('chabock_price', $price, time() + (86400 * 30), "/");
        wp_die(json_encode($price));
    }

    public function show_tracking_code_in_order($order)
    {
        $order_id = $order->get_id();
        global $PeproUltimateInvoice;
        $generator = $PeproUltimateInvoice->barcode;

        if (get_post_meta($order_id, 'order_tracking_chabok', true)) {
            if (is_plugin_active('pepro-ultimate-invoice/pepro-ultimate-invoice.php')) {
                $src = 'data:image/jpeg;base64,' . base64_encode($generator->getBarcode(get_post_meta($order_id, 'order_tracking_chabok', true), $generator::TYPE_CODE_128));
                echo '<strong>شماره بارنامه</strong><img alt="" class="barcode" style="width:150px;height:auto;display:block" src="' . $src . '"><span style="text-align:center">' . get_post_meta($order_id, 'order_tracking_chabok', true) . '</span>';
            } else {
                echo '<strong>شماره بارنامه</strong><span style="text-align:center">' . get_post_meta($order_id, 'order_tracking_chabok', true) . '</span>';
            }
        }
    }

    public function admin_scripts()
    {
        ?>
        <script>
            (function($) {
                $('#wc_chabok_store_city').change(function() {
                    var x = $(this).find(':selected').text();
                    $('#wc_chabok_store_city_name').val(x);
                    $('#wc_chabok_store_city_name').attr('value', x);
                });

                $('select#wc_chabok_store_state').change();
            })(jQuery);
        </script>
<?php
    }
}

$chabok_WC = new Chabok_WC;
